@@ -1,12 +1,50 @@
-`dbledensitymap` <-
-function (long, lat, var1,var2,kernel='triweight',listvar=NULL, listnomvar=NULL,carte=NULL,
-criteria=NULL, label="", cex.lab=1, pch=16, col=c("grey","blue"), xlab=c("",""),
-ylab=c("density","density"),axes=FALSE, lablong="", lablat="")
+`dbledensitymap` <- function(sp.obj, names.var, kernel='triweight',
+names.attr=names(sp.obj), criteria=NULL, carte=NULL, identify=FALSE, cex.lab=0.8, pch=16,
+col=c("grey","lightblue3"), xlab=c("",""), ylab="", axes=FALSE, lablong="", lablat="")
 {
+# Verification of the Spatial Object sp.obj
+class.obj<-class(sp.obj)[1]
+
+if(substr(class.obj,1,7)!="Spatial") stop("sp.obj may be a Spatial object")
+if(substr(class.obj,nchar(class.obj)-8,nchar(class.obj))!="DataFrame") stop("sp.obj should contain a data.frame")
+if(!is.numeric(names.var) & length(match(names.var,names(sp.obj)))!=length(names.var) ) stop("At least one component of names.var is not included in the data.frame of sp.obj")
+if(length(names.attr)!=length(names(sp.obj))) stop("names.attr should be a vector of character with a length equal to the number of variable")
+
+# Is there a Tk window already open ?
+if(interactive())
+{
+ if(!exists("GeoXp.open",envir = baseenv())||length(ls(envir=.TkRoot$env, all=TRUE))==2)
+ {
+  assign("GeoXp.open", TRUE, envir = baseenv())
+ }
+ else
+ {if(get("GeoXp.open",envir= baseenv()))
+   {stop("Warning : a GeoXp function is already open. Please, close Tk window before calling a new GeoXp function to avoid conflict between graphics")}
+  else
+  {assign("GeoXp.open", TRUE, envir = baseenv())}
+ }
+}
+
+# we propose to refind the same arguments used in first version of GeoXp
+long<-coordinates(sp.obj)[,1]
+lat<-coordinates(sp.obj)[,2]
+
+var1<-sp.obj@data[,names.var[1]]
+var2<-sp.obj@data[,names.var[2]]
+
+listvar<-sp.obj@data
+listnomvar<-names.attr
+
+# Code which was necessary in the previous version
+if(is.null(carte) & class.obj=="SpatialPolygonsDataFrame") carte<-spdf2list(sp.obj)$poly
+
+ # for identifyng the selected sites
+ifelse(identify, label<-row.names(listvar),label<-"")
+
   # initialisation
   obs<-vector(mode = "logical", length = length(long))
-  graph1<-"Densityplot1" 
-  graph2<-"Densityplot1"
+  graph1<-"Densityplot2"
+  graph2<-"Densityplot2"
   nointer<-FALSE
   nocart<-FALSE
   z<-NULL
@@ -14,10 +52,11 @@ ylab=c("density","density"),axes=FALSE, lablong="", lablat="")
   interv<-NULL
   buble<-FALSE
   legends<-list(FALSE,FALSE,"","")
-  graphics.off()
 
-  alpha11<-20 
-  alpha21<-20 
+  # for the slider
+  alpha11<-20
+  alpha21<-20
+  names.slide<-c("Alpha (1st graph)","Alpha (2nd graph)")
   
   graphChoice <- ""
   varChoice1 <- ""
@@ -30,7 +69,7 @@ ylab=c("density","density"),axes=FALSE, lablong="", lablat="")
   polyX2 <- NULL
   method <- ""
   col2 <- "blue"
-  col3 <- col[1]
+  col3<-"lightblue3"
   pch2<-pch[1]
   labmod<-""
   labvar1<-c(xlab[1],ylab[1])
@@ -40,71 +79,94 @@ ylab=c("density","density"),axes=FALSE, lablong="", lablat="")
 # Transformation de data.frame en matrix
 if((length(listvar)>0)&&(dim(as.matrix(listvar))[2]==1)) listvar<-as.matrix(listvar)
 
-# Ouverture des fentres graphiques
-dev.new()
-dev.new()
-dev.new()
-fin <- tclVar(FALSE)
+# Windows device
+if(!(2%in%dev.list())) dev.new()
+if(!(3%in%dev.list())) dev.new()
+if(!(4%in%dev.list())) dev.new()
 
 ####################################################
 # slection d'un point
 ####################################################
 
-pointfunc<-function() 
+pointfunc<-function()
 {
    if((graph1=="Densityplot2")||(graph2=="Densityplot2"))
-    { 
-     #SGfunc()
-      graph1<<-"Densityplot1" 
-      graph2<<-"Densityplot1" 
-      
-      graphique(var1=var1, obs=obs, alpha1=alpha11,  num=3, graph=graph1, labvar=labvar1,
-      couleurs=col[1],symbol=pch,kernel=kernel)
-     
-      graphique(var1=var2, obs=obs, alpha1=alpha21,  num=4, graph=graph2, labvar=labvar2,
-      couleurs=col[2],symbol=pch,kernel=kernel);    
-      
-    } 
-    
-    quit <- FALSE;
-    loc <- NULL;
-
-    while(!quit)
     {
-        dev.set(2);
-        loc<-locator(1);
-
-    graph1<<-"Densityplot1"  
-    graph2<<-"Densityplot1" 
-
-        if(is.null(loc)) 
-        {
-          quit<-TRUE
-          next
-        }
-
-        obs<<-selectmap(var1=long,var2=lat,obs=obs,Xpoly=loc[1], Ypoly=loc[2], method="point");         
-        
-        # graphiques
-      if (length(var1[obs]) >= 1)
+     #SGfunc()
+      graph1<<-"Densityplot1"
+      graph2<<-"Densityplot1"
+      
+       if (length(var1[obs]) > 1)
         {
          graphique(var1=var1, obs=obs, alpha1=alpha11,  num=3, graph=graph1, labvar=labvar1,
          couleurs=col[1],symbol=pch,kernel=kernel)
-     
+
          graphique(var1=var2, obs=obs, alpha1=alpha21,  num=4, graph=graph2, labvar=labvar2,
          couleurs=col[2],symbol=pch,kernel=kernel)
         }
+       else
+       {dev.set(3)
+       title(sub = "You have to choose at least two sites to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+       dev.set(4)
+       title(sub = "You have to choose at least two sites to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+     }
+    }
+
+    quit <- FALSE
+    loc <- NULL
+
+    dev.set(2)
+    title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+    title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
+    while(!quit)
+    {
+        dev.set(2)
+        loc<-locator(1)
+
+    graph1<<-"Densityplot1"
+    graph2<<-"Densityplot1"
+
+        if(is.null(loc))
+        {
+          quit<-TRUE
+        carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,
+        label=label,cex.lab=cex.lab, carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes,
+        lablong=lablong, lablat=lablat,symbol=pch2, couleurs=col2,method=method,
+        classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)
+          next
+        }
+
+        obs<<-selectmap(var1=long,var2=lat,obs=obs,Xpoly=loc[1], Ypoly=loc[2], method="point");
 
         carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,
         label=label,cex.lab=cex.lab, carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes,
         lablong=lablong, lablat=lablat,symbol=pch2, couleurs=col2,method=method,
         classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)
-        
+        title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+        title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
+        # graphiques
+      if (length(var1[obs]) > 1)
+        {
+         graphique(var1=var1, obs=obs, alpha1=alpha11,  num=3, graph=graph1, labvar=labvar1,
+         couleurs=col[1],symbol=pch,kernel=kernel)
+
+         graphique(var1=var2, obs=obs, alpha1=alpha21,  num=4, graph=graph2, labvar=labvar2,
+         couleurs=col[2],symbol=pch,kernel=kernel)
+        }
+       else
+       {dev.set(3)
+       title(sub = "You have to choose at least two sites to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+       dev.set(4)
+       title(sub = "You have to choose at least two sites to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+     }
+
 
          if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
           {graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
-            obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)        
-          }    
+            obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)
+          }
     }
   }
 
@@ -112,33 +174,36 @@ pointfunc<-function()
 # slection d'un polygone
 ####################################################
 
-polyfunc<-function() 
+polyfunc<-function()
 {
 
    if((graph1=="Densityplot2")||(graph2=="Densityplot2"))
-    { 
+    {
      #SGfunc()
-      graph1<<-"Densityplot1" 
-      graph2<<-"Densityplot1" 
-      
+      graph1<<-"Densityplot1"
+      graph2<<-"Densityplot1"
+
       graphique(var1=var1, obs=obs, alpha1=alpha11,  num=3, graph=graph1, labvar=labvar1,
       couleurs=col[1],symbol=pch,kernel=kernel)
-     
+
       graphique(var1=var2, obs=obs, alpha1=alpha21,  num=4, graph=graph2, labvar=labvar2,
-      couleurs=col[2],symbol=pch,kernel=kernel)    
-      
-    } 
-    
+      couleurs=col[2],symbol=pch,kernel=kernel)
+
+    }
+
     polyX <- NULL
     polyY <- NULL
     quit <- FALSE
 
+    dev.set(2)
+    title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+    title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
 
     while(!quit)
     {
         dev.set(2)
         loc<-locator(1)
-        if(is.null(loc)) 
+        if(is.null(loc))
         {
             quit<-TRUE
             next
@@ -147,7 +212,7 @@ polyfunc<-function()
         polyX <- c(polyX, loc[1])
         polyY <- c(polyY, loc[2])
         lines(polyX,polyY)
-    }
+  }
 
     polyX <- c(polyX, polyX[1])
     polyY <- c(polyY, polyY[1])
@@ -159,24 +224,33 @@ if (length(polyX)>0)
     obs <<- selectmap(var1=long, var2=lat, obs=obs, Xpoly=polyX, Ypoly=polyY, method="poly")
 
 
-    graphique(var1=var1, obs=obs, alpha1=alpha11,  num=3, graph=graph1, labvar=labvar1,
-    couleurs=col[1],symbol=pch,kernel=kernel)
-     
-    graphique(var1=var2, obs=obs, alpha1=alpha21,  num=4, graph=graph2, labvar=labvar2,
-    couleurs=col[2],symbol=pch,kernel=kernel)
-        
+      if (length(var1[obs]) > 1)
+        {
+         graphique(var1=var1, obs=obs, alpha1=alpha11,  num=3, graph=graph1, labvar=labvar1,
+         couleurs=col[1],symbol=pch,kernel=kernel)
+
+         graphique(var1=var2, obs=obs, alpha1=alpha21,  num=4, graph=graph2, labvar=labvar2,
+         couleurs=col[2],symbol=pch,kernel=kernel)
+        }
+       else
+       {dev.set(3)
+       title(sub = "You have to choose at least two sites to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+       dev.set(4)
+       title(sub = "You have to choose at least two sites to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+     }
+
 
     carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,
     label=label,cex.lab=cex.lab, carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes,
     lablong=lablong, lablat=lablat,symbol=pch2, couleurs=col2,method=method,
     classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)
-        
+
 
     if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
      {graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
-      obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)        
-     }    
-}  
+      obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)
+     }
+}
 
  }
 
@@ -190,46 +264,56 @@ inter1func<-function()
 
 #    quit <- FALSE;
 
-  if(graph1=="Densityplot1")
-   { 
+  if(graph1=="Densityplot1"||(graph1==graph2))
+   {
     SGfunc()
-    graph1<<-"Densityplot2"  
-    graph2<<-"Densityplot1" 
+    graph1<<-"Densityplot2"
+    graph2<<-"Densityplot1"
    }
-   
+
     polyX <- NULL
     n.inter<-length(polyX2)
-    
+
     while (length(polyX)<2)
     {
       dev.set(3)
+      title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+      title(sub = "Click two times to select an interval", cex.sub = 0.8, font.sub = 3,col.sub='red')
       loc<-locator(1)
       polyX <- c(polyX, loc[1])
     }
-    
+
     polyX2[[n.inter+1]]<<-polyX
 
-    obs<<-selectstat(var1=var1,obs=obs,Xpoly=polyX[1], Ypoly=polyX[2],method="Densityplot") 
-        
+    obs<<-selectstat(var1=var1,obs=obs,Xpoly=polyX[1], Ypoly=polyX[2],method="Densityplot")
+
     # graphiques
 
          graphique(var1=var1, obs=obs, alpha1=alpha11,   num=3, graph=graph1, Xpoly=polyX2,
          labvar=labvar1, couleurs=col[1], kernel=kernel)
+
+       if (length(var1[obs]) > 1)
+        {
+        graphique(var1=var2, obs=obs, alpha1=alpha21,  num=4, graph=graph2,
+        labvar=labvar2, couleurs=col[2], kernel=kernel)
          
-         graphique(var1=var2, obs=obs, alpha1=alpha21,  num=4, graph=graph2,
-         labvar=labvar2, couleurs=col[2], kernel=kernel)
-  
+        }
+      else
+       {dev.set(4)
+       title(sub = "You have to choose at least two sites to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+       }
+     
         carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,
         label=label,cex.lab=cex.lab, carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes,
         lablong=lablong, lablat=lablat,symbol=pch2, couleurs=col2,method=method,
         classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)
-        
+
 
     if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
       {graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
-       obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)        
-      }  
-      
+       obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)
+      }
+
   }
 
 ####################################################
@@ -238,43 +322,52 @@ inter1func<-function()
 
 inter2func<-function()
 {
-  if(graph2=="Densityplot1")
-   { 
+  if(graph2=="Densityplot1"||(graph1==graph2))
+   {
     SGfunc()
-    graph1<<-"Densityplot1"  
-    graph2<<-"Densityplot2" 
+    graph1<<-"Densityplot1"
+    graph2<<-"Densityplot2"
    }
 
     polyX <- NULL
     n.inter<-length(polyX2)
-    
+
     while (length(polyX)<2)
     {
       dev.set(4)
+      title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+      title(sub = "Click two times to select an interval", cex.sub = 0.8, font.sub = 3,col.sub='red')
       loc<-locator(1)
       polyX <- c(polyX, loc[1])
     }
-    
+
     polyX2[[n.inter+1]]<<-polyX
-    
-    obs<<-selectstat(var1=var2,obs=obs,Xpoly=polyX[1], Ypoly=polyX[2],method="Densityplot") 
-        
+
+    obs<<-selectstat(var1=var2,obs=obs,Xpoly=polyX[1], Ypoly=polyX[2],method="Densityplot")
+
     # graphiques
      graphique(var1=var2, obs=obs, alpha1=alpha21,  num=4, graph=graph2, Xpoly=polyX2,
      labvar=labvar2, couleurs=col[2], kernel=kernel)
-     
+
+     if (length(var2[obs]) > 1)
+     {
      graphique(var1=var1, obs=obs, alpha1=alpha11,  num=3, graph=graph1,
      labvar=labvar1, couleurs=col[1],kernel=kernel)
-  
+     }
+     else
+     {dev.set(3)
+      title(sub = "You have to choose at least two sites to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+     }
+     
      carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,
      label=label,cex.lab=cex.lab, carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes,
      lablong=lablong, lablat=lablat,symbol=pch2, couleurs=col2,method=method,
      classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)
-        
+
     if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
       {graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
-       obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)        
-      }   
+       obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)
+      }
 
  }
 
@@ -283,15 +376,19 @@ inter2func<-function()
 # Choisir une valeur sur le 1er graphique
 ####################################################
 
-choixvalue1 <- function() 
+choixvalue1 <- function()
 {
-  if(graph1=="Densityplot1")
-   { 
+  if(graph1=="Densityplot1"||(graph1==graph2))
+   {
     SGfunc()
-    graph1<<-"Densityplot2"  
-    graph2<<-"Densityplot1" 
+    graph1<<-"Densityplot2"
+    graph2<<-"Densityplot1"
    }
-   
+
+   dev.set(3)
+   title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+      
+      
   tt1<-tktoplevel()
   Name <- tclVar("1st value")
   Name2 <- tclVar("2nd value")
@@ -300,40 +397,47 @@ choixvalue1 <- function()
   tkgrid(tklabel(tt1,text="Please enter values"),entry.Name,entry.Name2)
 
   OnOK <- function()
-  { 
+  {
 	 value1 <- tclvalue(Name)
 	 value2 <- tclvalue(Name2)
 	 n.inter<-length(polyX2)
 	 tkdestroy(tt1)
-	 
+
    if(is.na(as.numeric(value1))||is.na(as.numeric(value2)))
     {
         tkmessageBox(message="Sorry, but you have to choose decimal values",icon="warning",type="ok");
     }
       else
     {
-    
+
     polyX2[[n.inter+1]]<<- c(as.numeric(value1),as.numeric(value2))
-    
+
     obs<<-selectstat(var1=var1,obs=obs,Xpoly=as.numeric(value1), Ypoly=as.numeric(value2),method="Densityplot")
-        
+
     # graphiques
     graphique(var1=var1, obs=obs, alpha1=alpha11,   num=3, graph=graph1, Xpoly=polyX2,
     labvar=labvar1, couleurs=col[1], kernel=kernel)
-         
+
+    if (length(var1[obs]) > 1)
+     {
     graphique(var1=var2, obs=obs, alpha1=alpha21,  num=4, graph=graph2,
     labvar=labvar2, couleurs=col[2], kernel=kernel)
-  
+    }
+    else
+    {dev.set(4)
+     title(sub = "You have to choose one more site to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+    }
+
     carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,
     label=label,cex.lab=cex.lab, carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes,
     lablong=lablong, lablat=lablat,symbol=pch2, couleurs=col2,method=method,
-    classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)     
+    classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)
 
     if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
       {graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
-       obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)        
-      }  
-    }   
+       obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)
+      }
+    }
   }
 
   OK.but <-tkbutton(tt1,text="   OK   ",command=OnOK)
@@ -347,15 +451,18 @@ choixvalue1 <- function()
 # Choisir une valeur sur le deuxime graphique
 ####################################################
 
-choixvalue2 <- function() 
+choixvalue2 <- function()
 {
-  if(graph2=="Densityplot1")
-   { 
+  if(graph2=="Densityplot1"||(graph1==graph2))
+   {
     SGfunc()
-    graph1<<-"Densityplot1"  
-    graph2<<-"Densityplot2" 
+    graph1<<-"Densityplot1"
+    graph2<<-"Densityplot2"
    }
-     
+
+  dev.set(4)
+  title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+      
   tt1<-tktoplevel()
   Name <- tclVar("1st value")
   Name2 <- tclVar("2nd value")
@@ -365,40 +472,47 @@ choixvalue2 <- function()
 
 
   OnOK <- function()
-  { 
+  {
 	 value1 <- tclvalue(Name)
 	 value2 <- tclvalue(Name2)
 	 n.inter<-length(polyX2)
 	 tkdestroy(tt1)
-	 
+
    if(is.na(as.numeric(value1))||is.na(as.numeric(value2)))
     {
         tkmessageBox(message="Sorry, but you have to choose decimal values",icon="warning",type="ok")
     }
    else
     {
-    
+
     polyX2[[n.inter+1]]<<- c(as.numeric(value1),as.numeric(value2))
-    
-    obs<<-selectstat(var1=var2,obs=obs,Xpoly=as.numeric(value1), Ypoly=as.numeric(value2),method="Densityplot") 
-        
+
+    obs<<-selectstat(var1=var2,obs=obs,Xpoly=as.numeric(value1), Ypoly=as.numeric(value2),method="Densityplot")
+
     # graphiques
      graphique(var1=var2, obs=obs, alpha1=alpha21,  num=4, graph=graph2, Xpoly=polyX2,
      labvar=labvar2, couleurs=col[2], kernel=kernel)
-     
+
+    if (length(var1[obs]) > 1)
+     {
      graphique(var1=var1, obs=obs, alpha1=alpha11,  num=3, graph=graph1,
      labvar=labvar1, couleurs=col[1],kernel=kernel)
-  
+     }
+     else
+     {dev.set(3)
+      title(sub = "You have to choose one more site to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+     }
+
      carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,
      label=label,cex.lab=cex.lab, carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes,
      lablong=lablong, lablat=lablat,symbol=pch2, couleurs=col2,method=method,
      classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)
-        
+
     if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
       {graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
-       obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)        
-      }   
-    }   
+       obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)
+      }
+    }
   }
 
 OK.but <-tkbutton(tt1,text="   OK   ",command=OnOK)
@@ -410,47 +524,57 @@ tkfocus(tt1)
 ####################################################
 # modification du alpha pour la courbe de densit
 ####################################################
-  
+
 
 refresh1.code<-function(...)
 {
- alpha11<<-slider1(no=1) 
- if(graph1=="Densityplot1") 
- {graphique(var1=var1, obs=obs, alpha1=alpha11, num=3, graph=graph1, labvar=labvar1, couleurs=col[1],kernel=kernel)}
+ res<-slider1(names.slide=names.slide,no=1)
+ alpha11<<-res$alpha11
+ alpha21<<-res$alpha21
+ 
+ if(graph1=="Densityplot1")
+ {   if (length(var2[obs]) > 1)
+       {graphique(var1=var1, obs=obs, alpha1=alpha11, num=3, graph=graph1, labvar=labvar1, couleurs=col[1],kernel=kernel)
+       }
+       else
+       {dev.set(3)
+       title(sub = "You have to choose at least two sites to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+      }
+  }
  else
  {graphique(var1=var1, obs=obs,alpha1=alpha11,  num=3, graph="Densityplot2", Xpoly=polyX2,
-  labvar=labvar1, couleurs=col[1],kernel=kernel)}
+  labvar=labvar1, couleurs=col[1],kernel=kernel)
  }
-
-
-####################################################
-# modification du alpha pour la courbe de densit
-####################################################
-  
-refresh2.code<-function(...)
-{
- alpha21<<-slider2(no=1) 
- if(graph2=="Densityplot1") 
-  {graphique(var1=var2, obs=obs, alpha1=alpha21, num=4, graph=graph2, labvar=labvar2,
-   couleurs=col[2],kernel=kernel)}
+ 
+ if(graph2=="Densityplot1")
+  {   if (length(var2[obs]) > 1)
+      {graphique(var1=var2, obs=obs, alpha1=alpha21, num=4, graph=graph2, labvar=labvar2,
+       couleurs=col[2],kernel=kernel)
+      }
+      else
+      {dev.set(4)
+       title(sub = "You have to choose at least two sites to represent the sub-density", cex.sub = 0.8, font.sub = 3,col.sub='red')
+      }
+  }
  else
   {graphique(var1=var2, obs=obs,alpha1=alpha21, num=4, graph="Densityplot2", Xpoly=polyX2,
    labvar=labvar2, couleurs=col[2],kernel=kernel)}
  }
+ 
 
 
 ####################################################
 # rafraichissement des graphiques
 ####################################################
 
-SGfunc<-function() 
+SGfunc<-function()
 {
-    obs<<-vector(mode = "logical", length = length(long));   
-    polyX2 <<- NULL 
-     
-    graphique(var1=var2, obs=obs, alpha1=alpha21,   num=4, graph="Densityplot1", 
+    obs<<-vector(mode = "logical", length = length(long));
+    polyX2 <<- NULL
+
+    graphique(var1=var2, obs=obs, alpha1=alpha21,   num=4, graph="Densityplot1",
     labvar=labvar2, couleurs=col[2], kernel=kernel)
-    
+
     graphique(var1=var1, obs=obs, alpha1=alpha11,   num=3, graph="Densityplot1", labvar=labvar1,
     couleurs=col[1], kernel=kernel)
 
@@ -458,11 +582,11 @@ SGfunc<-function()
     label=label,cex.lab=cex.lab, carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes,
     lablong=lablong, lablat=lablat,symbol=pch2, couleurs=col2,method=method,
     classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)
-        
+
     if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
       {graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
-       obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)        
-      }   
+       obs=obs, num=5, graph=graphChoice, symbol=pch, labvar=c(varChoice1,varChoice2),couleurs=col3)
+      }
 }
 
 
@@ -470,11 +594,21 @@ SGfunc<-function()
 # quitter l'application
 ####################################################
 
-quitfunc<-function() 
+quitfunc<-function()
 {
-  tclvalue(fin)<-TRUE
-  graphics.off()
-  tkdestroy(tt)
+    #tclvalue(fin)<<-TRUE
+    tkdestroy(tt)
+    assign("GeoXp.open", FALSE, envir = baseenv())
+   # assign("obs", row.names(sp.obj)[obs], envir = .GlobalEnv)
+}
+
+quitfunc2<-function()
+{
+    #tclvalue(fin)<<-TRUE
+    tkdestroy(tt)
+    assign("GeoXp.open", FALSE, envir = baseenv())
+    print("Results have been saved in last.select object")
+    assign("last.select", which(obs), envir = .GlobalEnv)
 }
 
 ####################################################
@@ -491,7 +625,7 @@ cartfunc <- function()
     }
    else
    {
-    tkmessageBox(message="Spatial contours have not been given",icon="warning",type="ok")    
+    tkmessageBox(message="Spatial contours have not been given",icon="warning",type="ok")
    }
 }
 
@@ -502,56 +636,45 @@ cartfunc <- function()
 ####################################################
 
 graphfunc <- function()
-{ 
+{
    if ((length(listvar) != 0) && (length(listnomvar) != 0))
     {
-        dev.off(5)
         choix <<- selectgraph(listnomvar,listgraph)
         varChoice1 <<- choix$varChoice1
         varChoice2 <<- choix$varChoice2
         graphChoice <<- choix$graphChoice
-           
+
         if ((graphChoice != "") && (varChoice1 != ""))
         {
-          if (((graphChoice == "Histogram")&&(!is.numeric(listvar[,which(listnomvar == varChoice1)])))||((graphChoice == "Scatterplot")&&((!is.numeric(listvar[,which(listnomvar == varChoice1)]))||(!is.numeric(listvar[,which(listnomvar == varChoice2)]))))) 
+          if (((graphChoice == "Histogram")&&(!is.numeric(listvar[,which(listnomvar == varChoice1)])))||((graphChoice == "Scatterplot")&&((!is.numeric(listvar[,which(listnomvar == varChoice1)]))||(!is.numeric(listvar[,which(listnomvar == varChoice2)])))))
            {
             tkmessageBox(message="Variables choosed are not in a good format",icon="warning",type="ok");
            }
           else
            {
             res1<-choix.couleur(graphChoice,listvar,listnomvar,varChoice1,legends,col,pch)
-            
+
             method <<- res1$method
             col2 <<- res1$col2
             col3 <<- res1$col3
             pch2 <<- res1$pch2
             legends <<- res1$legends
             labmod <<- res1$labmod
-                     
-            dev.new()
+
+            if(!(5%in%dev.list())) dev.new()
             graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
-            obs=obs, num=5, graph=graphChoice, couleurs=col3,symbol=pch, labvar=c(varChoice1,varChoice2));    
-            
+            obs=obs, num=5, graph=graphChoice, couleurs=col3, symbol=pch, labvar=c(varChoice1,varChoice2));
+
             carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,
             label=label,cex.lab=cex.lab, symbol=pch2, couleurs=col2,carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes,
             lablong=lablong, lablat=lablat,method=method,classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)
             }
-       }   
+       }
    }
    else
    {
     tkmessageBox(message="Variables (listvar) and their names (listnomvar) must have been given",icon="warning",type="ok");
-   }  
-}
-
-####################################################
-# quitter l'application
-####################################################
-
-quitfunc<-function() 
-{
-  tclvalue(fin)<-TRUE
-  tkdestroy(tt)
+   }
 }
 
 
@@ -559,20 +682,20 @@ quitfunc<-function()
 # Open a no interactive selection
 ####################################################
 
-fnointer<-function() 
+fnointer<-function()
 {
  if (length(criteria) != 0)
  {
    ifelse(!nointer,nointer<<-TRUE,nointer<<-FALSE)
    carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,
    label=label,cex.lab=cex.lab, symbol=pch2, couleurs=col2, carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes,
-   lablong=lablong, lablat=lablat,method=method,classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod) 
+   lablong=lablong, lablat=lablat,method=method,classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)
  }
  else
  {
   tkmessageBox(message="Criteria has not been given",icon="warning",type="ok")
  }
- 
+
 }
 
 ####################################################
@@ -582,27 +705,26 @@ fnointer<-function()
 fbubble<-function()
 {
   res2<-choix.bubble(buble,listvar,listnomvar,legends)
-  
+
   buble <<- res2$buble
   legends <<- res2$legends
   z <<- res2$z
   legmap <<- res2$legmap
-  
+
   carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,
   label=label,cex.lab=cex.lab, symbol=pch2, couleurs=col2, carte=carte,nocart=nocart,legmap=legmap,legends=legends,axis=axes,
-  lablong=lablong, lablat=lablat,method=method,classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)  
+  lablong=lablong, lablat=lablat,method=method,classe=listvar[,which(listnomvar == varChoice1)],labmod=labmod)
 
 }
 
 ####################################################
 # Reprsentation graphique
 ####################################################
-        
-graphique(var1=var2, obs=obs, alpha1=alpha21, num=4, graph="Densityplot1", labvar=labvar2,
-couleurs=col[2],kernel=kernel)
+graphique(var1=var2, obs=obs, alpha1=alpha21, num=4, graph=graph1, labvar=labvar2,
+couleurs=col[2],kernel=kernel,Xpoly=NULL)
 
-graphique(var1=var1, obs=obs, alpha1=alpha11, num=3, graph="Densityplot1", labvar=labvar1,
-couleurs=col[1],kernel=kernel)
+graphique(var1=var1, obs=obs, alpha1=alpha11, num=3, graph=graph2, labvar=labvar1,
+couleurs=col[1],kernel=kernel,Xpoly=NULL)
 
 carte(long=long, lat=lat,obs=obs,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,
 label=label,cex.lab=cex.lab, symbol=pch2, couleurs=col2, carte=carte,nocart=nocart,legmap=legmap,legends=legends,
@@ -615,100 +737,83 @@ axis=axes, lablong=lablong, lablat=lablat,method=method,classe=listvar[,which(li
 
 if(interactive())
 {
+fontheading<-tkfont.create(family="times",size=14,weight="bold")
+
 tt <- tktoplevel()
+tkwm.title(tt, "dbledensitymap")
 
-labelText1 <- tclVar("Work on the map")
-label1 <- tklabel(tt, text=tclvalue(labelText1))
-tkconfigure(label1, textvariable=labelText1)
-tkgrid(label1,columnspan=2)
-point.but <- tkbutton(tt, text="    Points    ", command=pointfunc);
-poly.but <- tkbutton(tt, text="   Polygon   ", command=polyfunc);
-tkgrid(point.but, poly.but, tklabel(tt,text="         "))
-tkgrid(tklabel(tt,text="    "))
+frame1a <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame1a, text = "Interactive selection", font = "Times 14",
+foreground = "blue", background = "white"))
+tkpack(tklabel(frame1a, text = "Work on the map", font = "Times 12",
+foreground = "darkred", background = "white"))
 
-labelText2 <- tclVar("Select an interval by selecting on graph")
-label2 <- tklabel(tt, text=tclvalue(labelText2))
-tkconfigure(label2, textvariable=labelText2)
-tkgrid(label2,columnspan=2)
-intervalle1.but <- tkbutton(tt, text="  1st graph  ", command=inter1func);
-intervalle2.but <- tkbutton(tt, text="  2nd graph  ", command=inter2func);
-tkgrid(intervalle1.but, intervalle2.but)
-tkgrid(tklabel(tt,text="    "))
+point.but <- tkbutton(frame1a, text="Selection by point", command=pointfunc);
+poly.but <- tkbutton(frame1a, text="Selection by polygon ", command=polyfunc);
+tkpack(point.but, poly.but, side = "left", expand = "TRUE",
+        fill = "x")
 
-labelText2 <- tclVar("Select an interval by specifying bounds")
-label2 <- tklabel(tt, text=tclvalue(labelText2))
-tkconfigure(label2, textvariable=labelText2)
-tkgrid(label2,columnspan=2)
-intervalle11.but <- tkbutton(tt, text="1st variable", command=choixvalue1);
-intervalle22.but <- tkbutton(tt, text="2nd variable", command=choixvalue2);
-tkgrid(intervalle11.but,intervalle22.but, tklabel(tt,text="         "))
-tkgrid(tklabel(tt,text="    "))
+tkpack(frame1a, expand = "TRUE", fill = "x")
+
+frame1c <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame1c, text = "Select an interval on the 1st graphic", font = "Times 12",
+foreground = "darkred", background = "white"))
+intervalle1.but <- tkbutton(frame1c, text="by selecting on graph", command=inter1func);
+intervalle11.but <- tkbutton(frame1c, text="by specifying bounds", command=choixvalue1);
+tkpack(intervalle1.but,intervalle11.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame1c, expand = "TRUE", fill = "x")
+
+frame1d <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame1d, text = "Select an interval on the 2nd graphic", font = "Times 12",
+foreground = "darkred", background = "white"))
+intervalle2.but <- tkbutton(frame1d, text="by selecting on graph", command=inter2func);
+intervalle22.but <- tkbutton(frame1d, text="by specifying bounds", command=choixvalue2);
+tkpack(intervalle2.but,intervalle22.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame1d, expand = "TRUE", fill = "x")
+
+frame1b <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+nettoy.but <- tkbutton(frame1b, text="     Reset selection     " , command=SGfunc);
+tkpack(nettoy.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame1b, expand = "TRUE", fill = "x")
 
 
+frame2 <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame2, text = "Options", font = "Times 14",
+foreground = "blue", background = "white"))
+tkpack(tklabel(frame2, text = "Spatial contours", font = "Times 10",
+foreground = "darkred", background = "white"),tklabel(frame2, text = "Preselected sites", font = "Times 10",
+foreground = "darkred", background = "white"),tklabel(frame2, text = "Bubbles", font = "Times 10",
+foreground = "darkred", background = "white"),tklabel(frame2, text = "Additional graph", font = "Times 10",
+foreground = "darkred", background = "white"), side = "left", fill="x",expand = "TRUE")
+tkpack(frame2, expand = "TRUE", fill = "x")
 
-label1 <- tclVar("To stop selection, leave the cursor on the active graph, click on the right button of the mouse and stop")
-label11 <- tklabel(tt,justify = "center", wraplength = "3i", text=tclvalue(label1))
-tkconfigure(label11, textvariable=label1)
-tkgrid(label11,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
+frame2b <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+nocou1.but <- tkbutton(frame2b, text="On/Off", command=cartfunc)
+noint1.but <- tkbutton(frame2b, text="On/Off", command=fnointer)
+bubble.but <- tkbutton(frame2b, text="On/Off", command=fbubble)
+autre.but <- tkbutton(frame2b, text="     OK     " , command=graphfunc)
+tkpack(nocou1.but,noint1.but,bubble.but,autre.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame2b, expand = "TRUE", fill = "x")
 
 
-slider1(tt,refresh1.code,c("Alpha (1st graph)"),c(3),c(100),c(1),c(alpha11))
+frame2e <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+slider1(frame2e,refresh1.code,names.slide,3,100,1,c(alpha11,alpha21))
+tkpack(frame2e, expand = "TRUE", fill = "x")
 
-slider2(tt,refresh2.code,c("Alpha (2nd graph)"),c(3),c(100),c(1),c(alpha21))
+frame3 <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame3, text = "Exit", font = "Times 14",
+foreground = "blue", background = "white"))
 
+quit.but <- tkbutton(frame3, text="Save results", command=quitfunc2);
+quit.but2 <- tkbutton(frame3, text="Exit without saving", command=quitfunc)
 
-labelText7 <- tclVar("Preselected sites / Draw Spatial Contours")
-label7 <- tklabel(tt, text=tclvalue(labelText7))
-tkconfigure(label7, textvariable=labelText7)
-tkgrid(label7,columnspan=2)
-noint1.but <- tkbutton(tt, text="  On/Off  ", command=fnointer);
-nocou1.but <- tkbutton(tt, text="  On/Off  ", command=cartfunc);
-tkgrid(noint1.but, nocou1.but)
-tkgrid(tklabel(tt,text="    "))
+tkpack(quit.but, quit.but2, side = "left", expand = "TRUE",
+        fill = "x")
 
-
-
-labelText9 <- tclVar("Bubbles")
-label9 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText9))
-tkconfigure(label9, textvariable=labelText9)
-tkgrid(label9,columnspan=2)
-
-bubble.but <- tkbutton(tt, text="  On/Off  ", command=fbubble);
-tkgrid(bubble.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-labelText3 <- tclVar("Restore graph")
-label3 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText3))
-tkconfigure(label3, textvariable=labelText3)
-tkgrid(label3,columnspan=2)
-
-nettoy.but <- tkbutton(tt, text="     OK     " , command=SGfunc);
-tkgrid(nettoy.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-labelText4 <- tclVar("Additional graph")
-label4 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText4))
-tkconfigure(label4, textvariable=labelText4)
-tkgrid(label4,columnspan=2)
-
-autre.but <- tkbutton(tt, text="     OK     " , command=graphfunc);
-tkgrid(autre.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-
-labelText5 <- tclVar("  Exit  ")
-label5 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText5))
-tkconfigure(label5, textvariable=labelText5)
-tkgrid(label5,columnspan=2)
-
-quit.but <- tkbutton(tt, text="     OK     ", command=quitfunc);
-tkgrid(quit.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-tkwait.variable(fin)
+tkpack(frame3, expand = "TRUE", fill = "x")
 }
 ####################################################
 
-return(obs)
-  }
+return(invisible())
+}
 
